#!/bin/bash
#
# EeePC audio output setup script
# Works with the Azalia driver
# Original version: 2008-10 by Quinn Storm <livinglatexkali@gmail.com>
# Modifications: 2008-11-03 by Jamie Curmi <jamie@curmi.com>
# Call with 0 for off, 1 for speakers, 2 for headphones

# Init code
init()
{
    # Set up verb address values
    REG_BASE=$(get_base_addr)
    if [ "$(echo $REG_BASE | cut -b 7-)" == "00" ]
    then
        BASE_ADDR=$(echo $REG_BASE | cut -b 1-6)
        VERB_DATA_ADDR=0x${BASE_ADDR}60
        VERB_CMD_ADDR=0x${BASE_ADDR}68
    else
        echo "ERROR -- Unaligned base address." 1>&2
    exit 1
    fi
}

# Get base address
get_base_addr()
{
    # This ugly line gets the base address, eliminating hand configuration
    ioreg -d 1 -r -n $(ioreg -x | grep 'AzaliaController' -B 1 | head -n 1 | cut -d 'o' -f 2- | cut -d '<' -f 1) -x | grep "IODeviceMemory" | cut -d '=' -f 3 | cut -d ',' -f 1 | cut -b 11-
}

# Send a verb to the azalia driver
send_verb()
{
    reggie_se -D PhysAddr -w "$1" -a ${VERB_DATA_ADDR} -B 32 &> /dev/null
    reggie_se -D PhysAddr -w 1 -a ${VERB_CMD_ADDR} -B 32 &> /dev/null
}

# Turn on internal speakers
enable_spk()
{
    send_verb 0x01470C02 #spk pin ctl
    send_verb 0x0143B000 #spk eapd on
    send_verb 0x0153B080 #hp eapd off 
}

# Turn off headphones
enable_hp()
{
    send_verb 0x015707C0 #hp pin ctl
    send_verb 0x0153B000 #hp eapd on
    send_verb 0x0143B080 #spk eapd off
}

# Turn off speaker and headphones
enable_off()
{
    send_verb 0x0143B080 #spk eapd off
    send_verb 0x0153B080 #hp eapd off
}

set_state()
{
    if [ "$1" == "0" ]
    then
        enable_off;
        return
    elif [ "$1" == "1" ]
    then
    	enable_spk;
        return
    elif [ "$1" == "2" ]
    then
    	enable_hp;
        return
    fi
}

init
set_state "$@"
